@echo off
setlocal enableextensions enabledelayedexpansion

REM *************************************************************************
REM * Submits a SAS job to a grid using sasgsub, waits for the job to
REM * finish and returns the RC of the job.
REM *
REM * For SAS versions 9.2m2 through 9.3m1
REM *
REM * Version 20121015
REM *************************************************************************

REM ***********************************************************************
REM  You may/must edit the values in this section
REM ***********************************************************************

REM Set 'debug' to 1 to get debug output
set debug=0

REM Edit the following variable to point to your SAS configuration directory
REM if you do not set the SASCONFIGDIR environment variable
REM
REM It should look something like
REM    sasconfigdir=C:\SAS\config\Lev1
if "%sasconfigdir%"=="" (
    set sasconfigdir=C:\config_dir\LevX
)

REM The number of seconds to wait between status checks. It should be at
REM least 5 seconds.
REM This is done for two reasons:
REM   1) The polling of the status should not be done too often or the grid
REM      performance may suffer
REM   2) There may be a delay between when a job is done and when the file
REM      system has updated its information
set waittime=15


REM ***********************************************************************
REM   !!! Do not edit below this line !!!
REM ***********************************************************************



REM ***********************************************************************
REM   Parse the parameters
REM ***********************************************************************

REM ---------------------------------------------
REM Initialize variables
set help=0
set sasgsubrc=999
set "metapass="
set "resultstr="
set "jobstr="
set "jnum="
set "TMPFILE="
set parms=%*

REM ---------------------------------------------
REM If none, spit out usage
if "%1"=="" (
    set help=1
    goto usage
)

REM ---------------------------------------------
REM If wanting usage, spit it out
if "%1"=="/?" (
    set help=1
    goto usage
)

REM ---------------------------------------------
REM Check that LSF is setup properly
if "%LSF_ENVDIR%"=="" (
    echo GSUBRC [%time%]: ERROR: The LSF environment does not appear to be setup.
    goto usage
)

if not exist %LSF_ENVDIR% (
    echo GSUBRC [%time%]: ERROR: The LSF environment does not appear to be setup.
    goto usage
)

REM ---------------------------------------------
REM Check that the config directory exists.
if not exist %sasconfigdir% (
    echo GSUBRC [%time%]: ERROR: The configuration directory "%sasconfigdir%" does not exist.
    goto usage
)

REM ---------------------------------------------
REM Check that the sasgsub directory exists.
set sasgsubdir=%sasconfigdir%\Applications\SASGridManagerClientUtility

if not exist %sasgsubdir% (
    echo GSUBRC [%time%]: ERROR: The SASGSUB directory "%sasgsubdir%" does not exist.
    goto usage
)

REM ---------------------------------------------
REM Find the version subdirectory to use
for /D %%X in (%sasgsubdir%\*) do (
        set sasgsubdir=%%X
    )
)

REM ---------------------------------------------
REM Check that the sasgsub directory exists.
if not exist %sasgsubdir% (
    echo GSUBRC [%time%]: ERROR: The SASGSUB directory "%sasgsubdir%" does not exist.
    goto usage
)

REM ---------------------------------------------
REM Check that SAS program exists.
if not exist %1 (
    echo GSUBRC [%time%]: ERROR: File "%1" not found.
    goto usage
)

REM ---------------------------------------------
REM Determine sas file name without the .SAS
set filename=%1
set filename=%filename%###
set filename=%filename:.SAS###=%
set filename=%filename:.sas###=%
set filename=%filename:###=%

REM ---------------------------------------------
REM see if one of the options is metapass. If so,
REM save the value. If not let the user know
REM SASGSUB may have issues if it is not defined
:checkpass
if "%1"=="" goto skipcheck

if "%1"=="-metapass" (
    set "metapass=-metapass %2"
    goto skipcheck
)

shift

goto checkpass

:skipcheck

if "%metapass%"=="" (
    echo GSUBRC [%time%]: METAPASS parameter not found. Assuming it is in SASGSUB.CFG.
    echo GSUBRC [%time%]: If METAPASS is not defined in SASGSUB.CFG, GSUBRC will hang.
)

REM ---------------------------------------------
REM SAS file exists. Generate two temp file names
:gettempfile
set TMPFILE=%TMP%\gbsubrc-%RANDOM%-%TIME:~6,5%.tmp
if exist "%TMPFILE%" goto :gettempfile
if %debug%==1 (
    echo GSUBRC [%time%]: Created TMPFILE %TMPFILE%
)

:gettempfile2
set TMPFILE2=%TMP%\gbsubrc-%RANDOM%-%TIME:~6,5%.tmp
if exist "%TMPFILE2%" goto :gettempfile2
if %debug%==1 (
    echo GSUBRC [%time%]: Created TMPFILE2 %TMPFILE2%
)


REM ***********************************************************************
REM   Submit the job
REM ***********************************************************************
echo GSUBRC [%time%]: Submitting job...

call %sasgsubdir%\sasgsub -gridsubmitpgm %parms% >%TMPFILE2%

REM ---------------------------------------------
REM Let user see output from SASGSUB
if %debug%==1 (
    type %TMPFILE2%
)

REM ---------------------------------------------
REM Search for job ID string in output
type %TMPFILE2% | findstr "<" > %TMPFILE%

REM ---------------------------------------------
REM Load the SASGSUB output to the JOBSTR env var
set /p jobstr=<%TMPFILE%

REM ---------------------------------------------
REM Use the first occurance of the <[0-9]+>
REM pattern to find the job number.
for /f "tokens=2 delims=<" %%a in ("%jobstr%") do set firstdelim=%%a
for /f "tokens=1 delims=>" %%a in ("%firstdelim%") do set jnum=%%a

REM ---------------------------------------------
REM Make sure we got a job number. If not, let
REM the user know and exit
if "%jnum%"=="" (
    echo GSUBRC [%time%]: ERROR: SASGSUB did not return a job number. SASGSUB output is
    type %TMPFILE2%
    set sasgsubrc=2
    goto usage
)

REM ***********************************************************************
REM   Loop getting the status of the job and waiting until we see
REM   Finished or Failed
REM ***********************************************************************

REM ---------------------------------------------
REM Tell user what job we are waiting for
echo GSUBRC [%time%]: Waiting for job %jnum% to complete...

set rc=1

REM ---------------------------------------------
REM Make sure the wait time is not less than
REM 5 seconds
if %waittime% LSS 5 (
    echo GSUBRC [%time%]: The wait time cannot be less than 5 seconds. Setting it to 5 seconds.
    set waittime=5
)
set runmsg=0

:loopstart

REM ---------------------------------------------
REM Try to grab the results
call %sasgsubdir%\sasgsub %metapass% -gridgetresults %jnum% >%TMPFILE2%

REM ---------------------------------------------
REM Let user see output from SASGSUB
if %debug%==1 (
    type %TMPFILE2%
)

REM ---------------------------------------------
REM Search for job ID string in output
type %TMPFILE2% | findstr "%jnum%" > %TMPFILE%

REM ---------------------------------------------
REM If the output contains the word "Finished"
set findresult=0
for /f "delims=" %%a in ('findstr /m /c:"Finished" %TMPFILE%') do set findresult=%%a
if %debug%==1 (
    echo GSUBRC [%time%]: Finished parsed output = "%findresult%"
)

if "%findresult%"=="%TMPFILE%" (

    set /p resultstr=<%TMPFILE%
    set resultstr=!resultstr:~-6,6!
    echo !resultstr! | findstr /c:" RC:" > %TMPFILE%
    set /p rcstr=<%TMPFILE%

    if %debug%==1 (
        echo GSUBRC [%time%]: Finished parsed data = "!resultstr!"
        echo GSUBRC [%time%]: Found RC: = "!rcstr!"
    )

    if not "!rcstr!"=="" (
        for /f "tokens=1* delims=:" %%a in ("!rcstr!") do (
            set rc=%%b
            echo GSUBRC [%time%]: JOB FINISHED. RC=%%b
        )
    ) else (
        set rc=0
        echo GSUBRC [%time%]: JOB FINISHED. Assuming RC=0
    )

    goto cleanup
)

REM ---------------------------------------------
REM If the output contains the word "Failed"
set findresult=0
for /f "delims=" %%a in ('findstr /m /c:"Failed" %TMPFILE%') do set findresult=%%a
if %debug%==1 (
    echo GSUBRC [%time%]: Failed parsed output = "%findresult%"
)

if "%findresult%"=="%TMPFILE%" (

    set /p resultstr=<%TMPFILE%
    set resultstr=!resultstr:~-6,6!
    echo !resultstr! | findstr /c:" RC:" > %TMPFILE%
    set /p rcstr=<%TMPFILE%

    if %debug%==1 (
        echo GSUBRC [%time%]: Finished parsed data = "!resultstr!"
        echo GSUBRC [%time%]: Found RC: = "!rcstr!"
    )

    if not "!rcstr!"=="" (
        for /f "tokens=1* delims=:" %%a in ("!resultstr!") do (
            set rc=%%b
            echo GSUBRC [%time%]: JOB FAILED. RC=%%b
        )
    ) else (
        set rc=999
        echo GSUBRC [%time%]: JOB FAILED. Assuming RC=999
    )

    goto cleanup
)

REM ---------------------------------------------
REM if grid job not found, (already failed)
set findresult=0
for /f "delims=" %%a in ('findstr /m /c:"directory" %TMPFILE%') do set findresult=%%a
if %debug%==1 (
    echo GSUBRC [%time%]: Directory parsed output = "%findresult%"
)

if "%findresult%"=="%TMPFILE%" (
    echo GSUBRC [%time%]: Cannot find job information to determine RC
    set rc=998
    goto cleanup
)

REM ---------------------------------------------
REM find size of %TMPFILE% to determine if empty
for %%a in (%TMPFILE%) do (
    set length=%%~za
)

REM ---------------------------------------------
REM If empty, let user know and retry otherwise
REM repeat
if %length%==0 (
    echo GSUBRC [%time%]: sasgsub returned null.
) else (
    if %runmsg%==0 (
        echo GSUBRC [%time%]: Grid job %jnum% is running.
        set runmsg=1
    )
)

REM ---------------------------------------------
REM Wait the specified seconds and continue
sleep %waittime%
goto loopstart


REM ***********************************************************************
REM   End of loop
REM ***********************************************************************
:cleanup

REM ---------------------------------------------
REM Copy the result files from subdir to current
REM dir subdir is id'd by most recently changed.
echo GSUBRC [%time%]: Copying %filename%.log and %filename%.lst to current directory.
for /f "delims=" %%x in ('dir /od /b *') do set recent=%%x

REM ---------------------------------------------
REM Make sure it's not a file, then copy/replace
REM one directory up
if not "%recent:~-4,1%"=="." (
    cd %recent%
    if exist %filename%.log copy /Y %filename%.log ..
    if exist %filename%.lst copy /Y %filename%.lst ..
    cd ..
)

if %debug%==1 (
    echo RC is %RC%
)

REM ***********************************************************************
REM   Exit script
REM ***********************************************************************
:usage

REM ---------------------------------------------
REM Spew the usage it requested
if %help%==1 (
    echo.
    echo gsubrc93: Submits a SAS program to the grid using SASGSUB v9.2M2 through 9.3M1
    echo             and waits for a return code
    echo.
    echo USAGE: gsubrc93 *.sas [SASGSUB options]
    echo    *.sas           - Required: a sas job file for sasgsub to submit to a grid.
    echo    sasgsub options - Optional: Any additional sasgsub options.
    echo.
    echo gsubrc93 will print the return code and set the return value of the script
    echo to that return code.
    echo.
)

REM ---------------------------------------------
REM Delete temp files if they exist and we are
REM not debugging
if not %debug%==1 (
    if not "%TMPFILE%"=="" (
        if exist "%TMPFILE%" (
            del /q %TMPFILE%
        )
    )

    if not "%TMPFILE2%"=="" (
        if exist "%TMPFILE2%" (
            del /q %TMPFILE2%
        )
    )
)

REM ---------------------------------------------
REM This special line allows us to pass a
REM variable from inside a setlocal/endlocal to
REM the outside since the entire line is parsed
REM and evaluated at once.
endlocal & set sasgsubrc=%rc%

REM ---------------------------------------------
REM Exit batch script returning the return code
exit /B %SASGSUBRC%
