#!/bin/bash
#****************************************************************************
#* Submits a SAS job to a grid using sasgsub, waits for the job to
#* finish and returns the RC of the job.
#*
#* For SAS version 9.3m2
#*
#* Version 20140109
#****************************************************************************

#**************************************************************************
# You may/must edit the values in this section
#**************************************************************************

# Set 'debug' to 1 to get debug output
debug=0

# Edit the following variable to point to your SAS configuration directory
# if you do not set the SASCONFIGDIR environment variable
#
# It should look something like
#    sasconfigdir="/opt/SAS/config/Lev1"
sasconfigdir=$SASCONFIGDIR
if [ -z "$sasconfigdir" ]
then
    sasconfigdir="/config_dir/LevX"
fi

# The number of seconds to wait after job completes and the files are
# returned. This may need to be adjusted depending on how quickly the
# shared file system updates its information
waittime=5

#**************************************************************************
#  !!! Do not edit below this line !!!
#**************************************************************************



#**************************************************************************
#   Functions
#**************************************************************************
usage()
{
    echo ""
    echo "gsubrc93m2: Submits a SAS program to the grid using SASGSUB v9.3M2"
    echo "            and waits for a return code"
    echo ""
    echo "USAGE: gsubrc93m2 *.sas [SASGSUB options]"
    echo "   *.sas           - Required: a sas job file for sasgsub to submit to a grid."
    echo "   sasgsub options - Optional: Any additional sasgsub options."
    echo ""
    echo "gsubrc93m2 will print the return code and set the return value of the script"
    echo "to that return code."
    echo ""
}


checkStatus()
{
    #------------------------------------------------------
    # Initialize return value
    rc=-1

    #------------------------------------------------------
    # Try to grab the results
    $sasgsubdir/sasgsub -metapass $metapass -gridgetresults $jnum >$tmpfile

    #------------------------------------------------------
    # Dump the output if debugging
    if [ "$debug" -eq "1" ]
    then
        cat $tmpfile
    fi

    #------------------------------------------------------
    # Did it start running?
    resultstr=`cat $tmpfile | grep -i "is Running"`
    if [ -n "$resultstr" ]
    then
        if [ $runmsg -eq 0 ]
        then
            echo "GSUBRC [`date \"+%T\"`]: Job $jnum is running."
            runmsg=1
            export runmsg
        fi
    fi

    #------------------------------------------------------
    # Did it finish?
    resultstr=`cat $tmpfile | grep -i "Finished"`
    if [ -n "$resultstr" ]
    then
        rc=`echo "$resultstr" | grep -o -P '(?<=:)[0-9]+$'`
        if [ -z "$rc" ]
        then
            echo "GSUBRC [`date \"+%T\"`]: JOB FINISHED. Assuming RC=0"
            rc=0
        else
            echo "GSUBRC [`date \"+%T\"`]: JOB FINISHED. RC=$rc"
        fi
    fi

    #------------------------------------------------------
    # Did it fail?
    resultstr=`cat $tmpfile | grep -i "Failed"`
    if [ -n "$resultstr" ]
    then
        rc=`echo "$resultstr" | grep -o -P '(?<=:)[0-9]+$'`
        if [ -z "$rc" ]
        then
            echo "GSUBRC [`date \"+%T\"`]: JOB FAILED. Assuming RC=999"
            rc=999
        else
            echo "GSUBRC [`date \"+%T\"`]: JOB FAILED. RC=$rc"
        fi
    fi

    #------------------------------------------------------
    # Did it fail some other way?
    resultstr=`cat $tmpfile | grep -i "directory"`
    if [ -n "$resultstr" ]
    then
        echo "GSUBRC [`date \"+%T\"`]: Cannot find job information to determine RC"
        rc=998
    fi

    export rc
}

#**************************************************************************
#   Parse the parameters
#**************************************************************************

#----------------------------------------------------------
# Initialize variables
help=0
rc=1
parms=$*
metapass=""

#----------------------------------------------------------
# Setup the debug file name
if [ "${USER}" = "" ]
then
  USER=`id | sed -e 's/.*(\(.*\)) gid=.*/\1/'`
  export USER
fi

tmpfile=/tmp/gsubrc.${USER}.$$.`date +%Y%m%d%H%M%S`.txt

#----------------------------------------------------------
# If no arguments were passed in
if [ $# -eq 0 ]
then
    usage
    exit 999
fi

#----------------------------------------------------------
# If user wanted help
if [ "$1" == "-?" ]
then
    usage
    exit 999
fi

#----------------------------------------------------------
# Check to see if LSF has been sourced
if [ -z "$LSF_ENVDIR" ]
then
    echo "GSUBRC [`date \"+%T\"`]: ERROR: The LSF profile has not been sourced."
    exit 999
fi

if [ ! -d $LSF_ENVDIR ]
then
    echo "GSUBRC [`date \"+%T\"`]: ERROR: The LSF profile has not been sourced."
    exit 999
fi

#----------------------------------------------------------
# If the configuration directory is not valid, exit
if [ ! -d $sasconfigdir ]
then
    echo "GSUBRC [`date \"+%T\"`]: ERROR: SAS Configuration directory \"$sasconfigdir\" is not valid."
    exit 999
fi

#----------------------------------------------------------
# If the SASGSUB directory is not valid, exit
sasgsubdir="$sasconfigdir/Applications/SASGridManagerClientUtility"

if [ ! -d $sasgsubdir ]
then
    echo "GSUBRC [`date \"+%T\"`]: ERROR: SASGSUB directory \"$sasgsubdir\" is not valid."
    exit 999
fi

#----------------------------------------------------------
# Find out the version of SASGSUB we should use.
currentdir=`pwd -P`
sasgsubdir="$sasgsubdir/9.?"
cd $sasgsubdir
sasgsubdir=`pwd -P`
cd $currentdir

#----------------------------------------------------------
# If the SASGSUB directory is not valid, exit
if [ ! -d $sasgsubdir ]
then
    echo "GSUBRC [`date \"+%T\"`]: ERROR: SASGSUB directory \"$sasgsubdir\" is not valid."
    exit 999
fi

#----------------------------------------------------------
# If the SAS program file does not exists, exit
if [ ! -e $1 ]
then
    echo "GSUBRC [`date \"+%T\"`]: ERROR: The SAS program file \"$1\" was not found."
    exit 999
fi

#----------------------------------------------------------
# Determine sas file name without the .SAS
filename=$1
filename=`echo $filename | sed -e 's/.SAS$//'`
filename=`echo $filename | sed -e 's/.sas$//'`

#----------------------------------------------------------
# See if one of the options is metapass. If so, save
# the value
while [ ! -z "$1" ]
do
    if [ "$1" = "-metapass" ]
    then
        metapass="-metapass $2"
    fi
    shift
done

if [ -z "$metapass" ]
then
    echo "GSUBRC [`date \"+%T\"`]: METAPASS parameter not found. Assuming it is in SASGSUB.CFG."
    echo "GSUBRC [`date \"+%T\"`]: If METAPASS is not defined in SASGSUB.CFG, GSUBRC will fail."
fi

#----------------------------------------------------------
# Let the user know the names of the temp files
if [ "$debug" -eq "1" ]
then
    echo "GSUBRC [`date \"+%T\"`]: Using tmpfile $tmpfile"
fi

#****************************************************************************
# Submit program
#****************************************************************************
echo "GSUBRC [`date \"+%T\"`]: Submitting job and waiting for it to complete..."
$sasgsubdir/sasgsub -gridwait -gridsubmitpgm $parms >$tmpfile

if [ "$debug" -eq "1" ]
then
    cat $tmpfile
fi

#----------------------------------------------------------
# Determine job number from sasgsub output
jnum=`cat $tmpfile | grep -o -P "(?<=<)[0-9]+(?=>)"`
if [ -z "$jnum" ]
then
    echo ""
    echo "GSUBRC [`date \"+%T\"`]: ERROR: SASGSUB did not return a job number. SASGSUB output is"
    cat $tmpfile
    rm -f $tmpfile
    exit 1
fi

echo "GSUBRC [`date \"+%T\"`]: Job $jnum has completed."

#****************************************************************************
# Get the results
#****************************************************************************
#----------------------------------------------------------
# Wait for the file system to update
if [ $waittime -gt 0 ]
then
    echo "GSUBRC [`date \"+%T\"`]: Waiting $waittime seconds for file system to update."
    sleep $waittime
fi

#----------------------------------------------------------
# Grab the results
checkStatus

#****************************************************************************
# Copy result files from subdir to currentdir
#****************************************************************************

# This should be the most recently modified file
moddir=`ls -t | head -1`

if [ -d "$moddir" ]
then
    cd $moddir
    if [ -e $filename.log ]
    then
        cp -f $filename.log ..
    fi
    if [ -e $filename.lst ]
    then
        cp -f $filename.lst ..
    fi
    cd ..
else
    echo "GSUBRC [`date \"+%T\"`]: Could not copy log files: Could not find SASGSUB output directory."
fi

#****************************************************************************
# Clean up
#****************************************************************************

if [ "$debug" -ne "1" ]
then
    if [ -e $tmpfile ]
    then
        rm -f $tmpfile
    fi
fi

exit $rc
