@echo off
setlocal
rem *********************************************************************
rem   runsas.cmd                                            17May2007
rem
rem   Shell script that will wrap a SAS program to allow it to be
rem   submitted to the grid via SAS/CONNECT
rem
rem   syntax:
rem    runsas <sas file name> <extra SAS options>
rem
rem *********************************************************************
rem
rem ==> The following must be done:
rem
rem   1) Make sure the SAS_HOME environment variable is set to the path
rem      of SAS 9.1.3 SP4 code installed on your computer.
rem
rem       Example:
rem        set SAS_HOME=C:\Program Files\SAS\SAS 9.1
rem
set SAS_HOME=C:\Program Files\SAS\SAS 9.1
rem
rem
rem   2) Make sure the SAS_META* environment variables are set correctly
rem      so that the SAS application server's logical grid server definition
rem      can be read out of the metadata.
rem
rem       Example:
rem        set SAS_METASERVER='mymetaserver.mycompany.com'
rem        set SAS_METAPORT=8561
rem        set SAS_METAREPOS='Foundation'
rem        set SAS_METAUSER='sasdemo'
rem        set SAS_METAPASS='sasdemo_pwd'
rem
set SAS_METASERVER=
set SAS_METAPORT=8561
set SAS_METAREPOS='Foundation'
set SAS_METAUSER=
set SAS_METAPASS=
rem
rem
rem   3) Make sure the SAS_GRIDSERVER environment variable is set to the
rem      application server name associated with the logical grid server
rem
rem       Example:
rem        set SAS_GRIDSERVER=SASMain
rem
set SAS_GRIDSERVER=
rem

set retcode=1

rem ---------------------------------------------------------------------
rem    Check environment variables
rem ---------------------------------------------------------------------

rem
rem Check to make SAS exists where the environment variable says it does
rem
if exist "%SAS_HOME%\sas.exe" goto SasExeOK
    echo ERROR: SAS_HOME environment variable does not point to location of sas.exe
    goto Finished

rem
rem Check to make sure the metadata server name environment variable is set
rem
:SasExeOK

if not "%SAS_METASERVER%"=="" goto SvrVarOK
    echo ERROR: SAS_METASERVER environment variable not defined
    goto Finished

rem
rem Check to make sure the metadata server user environment variable is set
rem
:SvrVarOK

if not "%SAS_METAUSER%"=="" goto UsrVarOK
    echo ERROR: SAS_METAUSER environment variable not defined
    goto Finished

rem
rem Check to make sure the metadata server pwd environment variable is set
rem
:UsrVarOK

if not "%SAS_METAPASS%"=="" goto PwdVarOK
    echo ERROR: SAS_METAPASS environment variable not defined
    goto Finished

rem
rem Check to make sure the SAS application server environment variable is set
rem
:PwdVarOK

if not "%SAS_GRIDSERVER%"=="" goto GridVarOK
    echo ERROR: SAS_GRIDSERVER environment variable not defined
    goto Finished


rem
rem Check to make sure a file name was passed in
rem
:GridVarOK

if not "%1"=="" goto NameOK
    echo ERROR: Missing name of sas program to run
    goto Finished


rem
rem Check to make sure the file exists
rem
:NameOK

if exist "%1" goto FileOK
    echo ERROR: SAS program file does not exist or is not readable
    goto Finished

rem ---------------------------------------------------------------------
rem    Create new SAS file and run it
rem ---------------------------------------------------------------------

rem
rem Filename is runsas-YYYY-MM-DD-HH-MM-SS-hh.sas
rem The %DATE% returns the date, %TIME% returns the time
rem %var:~start,len% will substring an environment variable starting at start for len characters.
rem
rem By the way, if the time is before 10AM, it prints out as " H:MM:SS:hh" so we need to take
rem care of the blank.
rem
:FileOK

if "%TIME:~0,1%"==" " goto Before10
    set tmpfilename=runsas-%DATE:~10%-%DATE:~4,2%-%DATE:~7,2%-%TIME:~0,2%-%TIME:~3,2%-%TIME:~6,2%-%TIME:~9,2%
    goto DoIt

:Before10
    set tmpfilename=runsas-%DATE:~10%-%DATE:~4,2%-%DATE:~7,2%-0%TIME:~1,1%-%TIME:~3,2%-%TIME:~6,2%-%TIME:~9,2%

rem
rem Create the grid enabled version of the program and run it
rem
:DoIt

    set sasfilename=%tmpfilename%.sas
    set logfilename=%tmpfilename%.log
    set lstfilename=%tmpfilename%.lst

    echo sas filename="%sasfilename%"
    echo log filename="%logfilename%"
    echo lst filename="%lstfilename%"

    rem Output the header indicating this is a generated file
    echo /* SAS program generated to run %1 on the grid */                     > %sasfilename%
    echo /*                                             */                    >> %sasfilename%

    rem Output the header that will specify the metadata server
    echo options metaserver=%SAS_METASERVER%;                                 >> %sasfilename%
    echo options metaport=%SAS_METAPORT%;                                     >> %sasfilename%
    echo options metarepository=%SAS_METAREPOS%;                              >> %sasfilename%
    echo options metauser=%SAS_METAUSER%;                                     >> %sasfilename%
    echo options metapass=%SAS_METAPASS%;                                     >> %sasfilename%

    rem Create the return code check macro
    echo %%macro checkretcode(mac, msg);                                      >> %sasfilename%
    echo  %%if ^(^&mac ne 0^) %%then %%do;                                    >> %sasfilename%
    echo    %%let syscc=^&mac;                                                >> %sasfilename%
    echo    %%put;%%put;%%put RUNSAS ERROR: ^&msg, rc=^&mac;%%put;            >> %sasfilename%
    echo    endsas;                                                           >> %sasfilename%
    echo  %%end;                                                              >> %sasfilename%
    echo %%mend checkretcode;                                                 >> %sasfilename%

    rem Initialize the grid environment
    echo %%let rc=%%sysfunc(grdsvc_enable(grid, resource=%SAS_GRIDSERVER%));  >> %sasfilename%

    rem Check the grid enable return code
    echo %%checkretcode(^&rc,"Failed to enable grid");                        >> %sasfilename%

    rem Signon to the grid machine
    echo signon grid signonwait=yes cmacvar=smac;                             >> %sasfilename%

    rem Check the signon return code
    echo %%checkretcode(^&smac,"Grid signon failed");                         >> %sasfilename%

    rem Start an rsubmit
    echo rsubmit grid connectwait=yes cmacvar=rmac;                           >> %sasfilename%

    rem Read in the user's SAS program and output it to the end of the new
    rem sas file without any lines that have 'ENDSAS'
    findstr /I /V endsas %1                                                   >> %sasfilename%

    rem Append a terminating semicolon, run and quit just
    rem in case stuff was left hanging
    echo ; run; quit;                                                         >> %sasfilename%

    rem Send the completion code back to the client
    echo %%sysrput runsasrc=^&syscc;                                          >> %sasfilename%

    rem End the rsubmit and signoff
    echo endrsubmit;                                                          >> %sasfilename%
    echo signoff;                                                             >> %sasfilename%

    rem Check the rsubmit return code
    echo %%checkretcode(^&rmac,"Grid code submission failed");                >> %sasfilename%

    rem Check the user's code return code
    echo %%checkretcode(^&runsasrc,"User code failed");                       >> %sasfilename%

    rem End SAS
    echo endsas;                                                              >> %sasfilename%

    rem Set the SASJOBNAME parameter to the name of the sas file in case the
    rem user wants an esub to change the grid job name to the name of the SAS
    rem file
    set SASJOBNAME=%1

    rem Put all extra parameters in the SASGRIDOPTS environment variable
    rem so that an esub can be used to append to the grid command
    set extraopts=

   :ChkOpts
       shift
       if [%1]==[] goto SetOpts
       set extraopts=%extraopts% %1
       goto ChkOpts

   :SetOpts
       set SASGRIDOPTS=%extraopts%

    rem Invoke SAS
    "%SAS_HOME%\sas.exe" %sasfilename% -log %logfilename% -altprint %lstfilename%

    rem Save return code to be returned at end of script
    set retcode=%ERRORLEVEL%

    rem Remove generated SAS file
    erase /f /q %sasfilename%

:Finished

echo retcode=%retcode%

endlocal

exit /b %retcode%
