#!/bin/sh -x
# *********************************************************************
#   runsas.sh                                             17May2007
#
#   Shell script that will wrap a SAS program to allow it to be
#   submitted to the grid via SAS/CONNECT
#
#   syntax:
#      ./runsas.sh <sas file name> <extra SAS options>
#
# *********************************************************************
#
# ==> The following must be done:
#
#   1) Make sure the SAS_HOME environment variable is set to the path
#      of SAS 9.1.3 SP4 code installed on your computer.
#
#       Example:
#        SAS_HOME=/usr/local/SAS/SAS_9.1;
#
SAS_HOME=/usr/local/SAS/SAS_9.1
#
#
#   2) Make sure the SAS_META* environment variables are set correctly
#      so that the SAS application server's logical grid server definition
#      can be read out of the metadata.
#
#       Example:
#        SAS_METASERVER='mymetaserver.mycompany.com';
#        SAS_METAPORT=8561;
#        SAS_METAREPOS='Foundation';
#        SAS_METAUSER='sasdemo';
#        SAS_METAPASS='sasdemo_pwd';
#
SAS_METASERVER=
SAS_METAPORT=8561
SAS_METAREPOS='Foundation'
SAS_METAUSER=
SAS_METAPASS=
#
#
#   3) Make sure the SAS_GRIDSERVER environment variable is set to the
#      application server name associated with the logical grid server
#
#       Example:
#        SAS_GRIDSERVER=SASMain;
#
SAS_GRIDSERVER=
#

retcode=1;

# ---------------------------------------------------------------------
#    Check environment variables
# ---------------------------------------------------------------------

#
# Check to make SAS exists where the environment variable says it does
#
if [ ! -r "${SAS_HOME}/sas" ] ; then
    echo "ERROR: SAS_HOME environment variable does not point to the location of sas"

#
# Check to make sure the metadata server name environment variable is set
#
elif [ "${SAS_METASERVER}" = "" ] ; then
    echo "ERROR: SAS_METASERVER environment variable not defined"

#
# Check to make sure the metadata server user environment variable is set
#
elif [ "${SAS_METAUSER}" = "" ] ; then
    echo "ERROR: SAS_METAUSER environment variable not defined"

#
# Check to make sure the metadata server pwd environment variable is set
#
elif [ "${SAS_METAPASS}" = "" ] ; then
    echo "ERROR: SAS_METAPASS environment variable not defined"

#
# Check to make sure the SAS application server environment variable is set
#
elif [ "${SAS_GRIDSERVER}" = "" ] ; then
    echo "ERROR: SAS_GRIDSERVER environment variable not defined"

#
# Check to make sure a file name was passed in
#
elif [ "$1" = "" ] ; then
    echo "ERROR: Missing name of sas program to run"

#
# Check to make sure the file exists
#
elif [ ! -r "$1" ] ; then
    echo "ERROR: SAS program file does not exist or is not readable"

# ---------------------------------------------------------------------
#    Create new SAS file and run it
# ---------------------------------------------------------------------
else
    dateinfo=`date +%Y.%m.%d.%H.%M.%S`;
    prefilename=runsas.$dateinfo.$$;
    sasfilename=${prefilename}.sas;
    logfilename=${prefilename}.log;
    lstfilename=${prefilename}.lst;

    echo sas filename="${sasfilename}";

    # Output the header indicating this is a generated file
    echo "/* SAS program generated to run $1 on the grid */"                   > ${sasfilename}
    echo                                                                      >> ${sasfilename}

    # Output the header that will specify the metadata server
    echo "options metaserver=${SAS_METASERVER};"                              >> ${sasfilename}
    echo "options metaport=${SAS_METAPORT};"                                  >> ${sasfilename}
    echo "options metarepository=${SAS_METAREPOS};"                           >> ${sasfilename}
    echo "options metauser=${SAS_METAUSER};"                                  >> ${sasfilename}
    echo "options metapass='${SAS_METAPASS}';"                                >> ${sasfilename}

    # Create the return code check macro
    echo "%macro checkretcode(mac, msg);"                                     >> ${sasfilename}
    echo " %if (&mac ne 0) %then %do;"                                        >> ${sasfilename}
    echo "   %let syscc=&mac;"                                                >> ${sasfilename}
    echo "   %put;%put;%put RUNSAS ERROR: &msg, rc=&mac;%put;"                >> ${sasfilename}
    echo "   endsas;"                                                         >> ${sasfilename}
    echo " %end;"                                                             >> ${sasfilename}
    echo "%mend checkretcode;"                                                >> ${sasfilename}

    # Initialize the grid environment
    echo "%let rc=%sysfunc(grdsvc_enable(grid, resource=${SAS_GRIDSERVER}));" >> ${sasfilename}

    # Check the grid enable return code
    echo "%checkretcode(&rc,""Failed to enable grid"");"                      >> ${sasfilename}

    # Signon to the grid machine
    echo "signon grid signonwait=yes cmacvar=smac;"                           >> ${sasfilename}

    # Check the signon return code
    echo "%checkretcode(&smac,""Grid signon failed"");"                       >> ${sasfilename}

    # Start an rsubmit
    echo "rsubmit grid connectwait=yes cmacvar=rmac;"                         >> ${sasfilename}

    # Read in the user's SAS program and output it to the end of the new
    # sas file without the ENDSAS command (mixing all cases of letters).
    sed 's/[eE][nN][dD][sS][aA][sS]//g' $1                                    >> ${sasfilename}

    # Append a terminating semicolon, run and quit just
    # in case stuff was left hanging
    echo "; run; quit;"                                                       >> ${sasfilename}

    # Send the completion code back to the client
    echo "%sysrput runsasrc=&syscc;"                                          >> ${sasfilename}

    # End the rsubmit and signoff
    echo "endrsubmit;"                                                        >> ${sasfilename}
    echo "signoff;"                                                           >> ${sasfilename}

    # Check the rsubmit return code
    echo "%checkretcode(&rmac,""Grid code submission failed"");"              >> ${sasfilename}

    # Check the user's code return code
    echo "%checkretcode(&runsasrc,""User code failed"");"                     >> ${sasfilename}

    # End SAS
    echo "endsas;"                                                            >> ${sasfilename}

    # Set the SASJOBNAME parameter to the name of the sas file in case the
    # user wants an esub to change the grid job name to the name of the SAS
    # file
    export SASJOBNAME=$1

    # Put all extra parameters in the SASGRIDOPTS environment variable
    # so that an esub can be used to append to the grid command
    #   (I do not use sed because the parameters may include stuff that
    #    confuses sed...)
    extraopts=
    shift

    if [ "$1" = "-log" ] ; then
      shift
      logfilename=$1;
      shift
    fi

    if [ "$1" = "-print" ] ; then
      shift
      lstfilename=$1;
      shift
    fi    
    
    echo log filename="${logfilename}";
    echo lst filename="${lstfilename}";

    while [ -n "$1" ]
      do
        extraopts="${extraopts} $1"
        shift
      done

    export SASGRIDOPTS="${extraopts}";

    # Invoke SAS
    ${SAS_HOME}/sas ${sasfilename} -log ${logfilename} -altprint ${lstfilename}

    # Save return code to be returned at end of script
    retcode=$?;

    # Remove generated SAS file
    rm -f ${sasfilename}

fi

echo retcode=${retcode}

exit ${retcode}
